uniform float specularIntensity, specularExponent;
uniform sampler2D TexDiffuse, TexNormal;
uniform bool doDiffuse, doShading, doNormal, doParallax;
varying vec3 eyeDirectionFS, lightDirectionFS, normalDirectionFS;
varying vec4 ambientLightColor, diffuseLightColor;

void main(void)
{
	vec3 eyeDirection = normalize(eyeDirectionFS);
	vec3 lightDirection = normalize(lightDirectionFS);
	vec2 TexCoord;
	vec3 normalDirection;
	vec4 diffuseColor;
	vec4 finalAmbient;
	vec4 finalDiffuse;
	vec4 finalSpecular;
	
	if (doParallax)
	{
		float height = .01 * texture2D(TexNormal, gl_TexCoord[0].xy).a - .005;
		TexCoord = gl_TexCoord[0].xy + height * eyeDirection.xy;
	}
	else
		TexCoord = gl_TexCoord[0].xy;
		
	if (doDiffuse)
		diffuseColor = texture2D(TexDiffuse, TexCoord);
	else
		diffuseColor = vec4(.6,.6,.6,1.0);

	if (doNormal)
		normalDirection = 2.0 * texture2D(TexNormal, TexCoord).xyz - 1.0;
	else
		normalDirection = normalize(normalDirectionFS);

	finalAmbient = ambientLightColor * diffuseColor;

	if (doShading)
	{
		float nDotL = max(dot(normalDirection, lightDirection), 0.0); 
		vec3 r = normalize(reflect(lightDirection, normalDirection));
		float rDotV = max(0.0, dot(r, eyeDirection));
		finalDiffuse = diffuseLightColor * nDotL * diffuseColor; 
		finalSpecular = gl_LightSource[0].specular * specularIntensity * pow(rDotV, specularExponent);
		gl_FragColor = finalAmbient + finalDiffuse + finalSpecular;
	}
	else
		gl_FragColor = finalAmbient;
}